## r-tutorial-1.R

# The pound sign indicates that the line you are reading is a comment. 
# You can tell this at a glance because RStudio automatically colors comments
# green. You should read all of the comments in the file. 

####################################
# PART I: R as a simple calculator #
#################################### 
# Put your cursor on the next line. Hit ctrl-enter.
1+1

# In the console below this screen you should see: 
# > 1+1
# [1] 2
# >

# Try something that looks a bit more like algebra. Again, put your 
# cursor on the next line and hit ctrl-enter. 
x <- 1+3

# This says: add 1 plus 3 and store the result in x. 
# What you should see is: 
# > x <- 1+3
# > 
# Note that in this case, R doesn't say anything on the console. No output
# usually means things worked as expected. 

# So how do we access the value of x? 
# Try hitting ctrl-enter on the next line.
x 

# This gets you: 
# > x 
# [1] 4

# Now R implements the standard PEMDAS order of operations. Note that
# the two expressions are not equivalent to each other. (You know what 
# to do from now on with ctrl-enter.)
x^2/4
x^(2/4)

#####################################
# PART II: R as a matrix calculator #
##################################### 
# R can also work with vectors and matrices.
# Let's generate a vector that counts from 1 to 10. 
1:10

# That gives you
# > 1:10
# [1]  1  2  3  4  5  6  7  8  9 10

# Let's store that in a variable:
vectorA <- 1:10

# Predict what the following expression will do: 
vectorA^2

# Did you get it right? 
# Answer: Each element in the vector was squared. 

# This also works for more complicated expressions: 
vectorA^2+3*vectorA-12

# And there are a bunch of functions that operate on 
# vectors to reduce them in an expected way. For example:
sum(vectorA)
mean(vectorA)
sd(vectorA)   #Standard Deviation
var(vectorA)  #VARiance
prod(vectorA) #PRODuct

# And a very useful one, which output several summary statistics at once
summary(vectorA)

# Matrices work the way you would expect them to:
# You can either fill them by column (the default)
matrixA <- matrix( 1:100, nrow=10, ncol=10)
matrixA

# Or by row: 
matrixB <- matrix( 1:100, nrow=10, ncol=10, byrow=TRUE)
matrixB

# And your standard matrix multiplication looks like this:
matrixA %*% matrixB 

# Note that %*% is the R operator for matrix multiplication. 
# You use it to multiply vectors and matrices in the expected
# way:
matrixA %*% vectorA

# If you forget about the percent signs you get element-wise
# multiplication instead of the matrix multiplication you 
# expected.
matrixA * vectorA

####################################
# PART III: Using R with real data #
#################################### 
# So far we haven't done much interesting because we've lacked
# the context of a real example. Let's get some data into R,
# clean it, and then explore it. 

# ***************************************************
# * Step 1: Getting Data into R and manipulating it *
# ***************************************************

# There are two ways to do this. Read both sets of 
# instructions and pick the one you like. 

# First way: A series of menu clicks
# ----------------------------------
# 0. Copy these steps to Notepad or Stickies or some other text program so 
#    that you can look at them while messing with RStudio. 
# 1. Look at the File, Edit, View ... menu at the top of the RStudio screen. 
# 2. Go to Workspace -> Import Dataset -> From file
# 3. Find the "thermo11_data_integrated.csv" file wherever you saved
#    it on your filesystem. Open it. 
# 4. In the box that popped up: 
#    A) In the name box, change 'thermo11_data_integrated' to 'thermo11'. 
#       This will be the name of the variable holding the dataset.
#    B) Look at the "Input File" and "Data Frame" sub-windows. Notice that
#       the "Data Frame" sub-window has V1, V2, V3, etc. above the headings.
#       Fix this by setting "Heading" to yes on the left side of the window.
#   C) Click import.
# 5. You should be back at RStudio's main window, but now the tabbed editor
#    is on a spreadsheet view of 'thermo11'. This looks kind of like Excel.
#    Check out all of the information we've got. 
# 6. Click back to the 'r-tutorial-1.R' tab so you can keep going. 

# End first way.

# Second way: A single command
# ----------------------------
thermo11 <- read.csv(file='thermo11_data_integrated.csv', header=TRUE)

# The file= arguement specifies the name of the file, the header= argument
# says that the first line has the variable names. 
#
# This second way is more useful, but you have to be careful about what 
# directory R is working in and where your files are. See the getwd and
# setwd commands for details. 

# End second way.

# If you just type thermo11, you'll get a lot of information printed to the 
# console that won't really help you. 
thermo11 

# If you want to look at a spreadsheet representation of the data, you can 
# use the following command. NOTE: This will open up a new tab which will hide
# these instructions. You'll need to click on the 'r-tutorial-1.R' tab to get 
# back. 
View(thermo11)

# If you are interested in a particular cell, you might want to just look
# at the entry in the first row of the seventh column
thermo11[1,7]

# And maybe you want to look at the first ten rows of the seventh column
thermo11[1:10,7]

# And maybe you want to look at the first ten rows of the columns seven to nine
thermo11[1:10,7:9]

# But counting the columns are kind of awkward, so you might just want to do
# it by the names. So let's build a vector of names using the c (Concatenate)
# command: 
c('ConditionTwo','ConditionThree','OrigSAmedian')

# And then drop it into the column part of the square brackets 
thermo11[1:10,c('ConditionTwo','ConditionThree','OrigSAmedian')]

# But maybe we only want the rows that meet a certain condition. Say
# that ConditionThree is targetHigh. So let's build a vector that has
# only the row numbers which meet the condition (instead of just 1:10). 
# A few steps to get there: 
# 1. Access the whole ConditionThree column. (The spacing is intentional
#    so that the steps are a little more obvious. Go through each one 
#    hitting ctrl-enter. )
                            thermo11$ConditionThree
# 2. Test if that column is equal to targetHigh
                            thermo11$ConditionThree == 'targetHigh' 
# 3. Get the indices of which rows are TRUE. 
                     which( thermo11$ConditionThree == 'targetHigh' )
# 4. Save it to a vector
select_targetHigh <- which( thermo11$ConditionThree == 'targetHigh' )

# Now we can look at just the targetHigh rows: 
thermo11[select_targetHigh,c('ConditionTwo','ConditionThree','OrigSAmedian')]


# **********************************
# * Step 2: Cleaning the data in R *
# **********************************

# So let's explore this data some. First let's see the kinds of data we have
# with the STRucture command: 
str(thermo11)

# You get lines that are all variations of the following three themes: 
#$ Section      : Factor w/ 6 levels "A","B","C","D",..: 1 1 1 1 1 1 1 1 1 1 ...
#$ PowerGoal    : int  0 0 1 1 0 1 0 1 1 0 ...
#$ preSEmastery : num  6.5 6.5 6 5.75 7 7 7 6 6.25 

# Section is a Factor. Factor is the datatype for categorical data. R calls 
#         the different categories levels. In this case we have A, B, C, etc.
# PowerGoal is an integer. It's only allowed to be a whole number. 
# preSEmastery is a number. R thinks of 'number' as values which are written
#         with a decimal. 

# Now note that although PowerGoal, Female, and Target are all currently 
# stored as integers, they are *really* categories. Let's change them to 
# factors so that R can make our life easier. 

# We are going to do this in a series of steps. To help you see what we
# are doing I've changed the spacing on the following lines so that you 
# can see what I'm changing: 

# If we just want to access a single column of the whole "spread sheet" 
# (R calls it a data frame), you use the $ operator like so: 
                                          thermo11$PowerGoal 

# So we want to change each 0 to a FALSE and each 1 to a TRUE. That's easy
# by asking R to convert the data to it logical type: 
                              as.logical( thermo11$PowerGoal )

# But what we really want is for TRUE and FALSE to be levels of a factor:
                      factor( as.logical( thermo11$PowerGoal ), levels=c(TRUE, FALSE))

# Notice how at the very bottom, we have 'Levels: TRUE FALSE' which tells
# us that it worked. 

# So the last step is just to write back INTO the PowerGoal column of the 
# thermo11 dataframe
thermo11$PowerGoal <- factor( as.logical( thermo11$PowerGoal ), levels=c(TRUE, FALSE))

# And we just repeat the recipe for Female and Target
thermo11$Female <- factor( as.logical( thermo11$Female ), levels=c(TRUE, FALSE))
thermo11$Target <- factor( as.logical( thermo11$Target ), levels=c(TRUE, FALSE))


#########################################
# Part IV: Exploring the data visually  #
#########################################

# So let's actually look at each of these kinds of data really quickly. 
# *****************************
# * Factors: Categorical data *
# *****************************

# One thing you might want to know is if Females were properly balanced in 
# the experimental design. If they were, we would expect 'Female' to be
# independent of 'PowerGoal' and 'Target'. 

# To do this, we need to take our spreadsheet of data: 
thermo11[,c('Female','PowerGoal','Target')]

# And turn it into a contingency table (pivot table for the Excel people).
# The R command for this is xtabs (CROSS tabulation). You tell it where 
# your data is with the data= option, and then you specify which columns to
# cross tabulate with an odd looking formula syntax.
femalePower <- xtabs( ~ Female + PowerGoal, data=thermo11)
femalePower

# And just as easily we could look at Female and Target
femaleTarget <- xtabs( ~ Female + Target, data=thermo11)
femaleTarget

# And we can also look at them both at the same time with a 
# three dimensional table
femalePowerTarget <- xtabs( ~ Female + PowerGoal + Target, data=thermo11) 
ftable(femalePowerTarget)

# NOTE: ftable just formats the output of the xtabs command a little nicer.

# Now all three of these tables are hard to LOOK at and see if they are 
# independent. So, let's use a tool called a mosaic plot. 

# If you are unfamiliar with a mosaic plot, the basic idea is that 
# if the "breaks" line up vertically and horizontally, the variables are
# independent. If not, they are dependent. See
#
#   http://www.childrensmercy.org/stats/definitions/images/mosaic2.gif
#
#   for a picture of how Sex and Survival status interacted for passengers
#   on the Titanic. Note how this brings out the "women first" mentality
#   on the lifeboats. 
#
# Similarly, see 
#
#   http://www.childrensmercy.org/stats/definitions/images/mosaic3.gif
#
#   for a picture of how Sex and Survival status WOULD look if the two
#   variables were independent. You can think of this as a "everyone 
#   for themselves" attitude. 
#
#   If you want more info about mosaic plots, take a look at the larger
#   tutorial I stole the images from: 
#
#   http://www.childrensmercy.org/stats/definitions/mosaic.htm1
#

# Now R can do mosaic plots, but the basic version of R can't. You need
# to install a package to use this feature. It's easy.

# Step 1: Do this once. Install the VCD package. 
install.packages('vcd', repos='http://lib.stat.cmu.edu/R/CRAN/')

# Step 2: Load the vcd package into the R session. 
# You have to do this each time you want to use it. 
library(vcd)

# So now we can make our own mosaic plot for Female and PowerGoal:
mosaic( femalePower )

# So it looks like the "break" lines do line up suggesting independence. 
# Let's do a formal statistical test to check this intuition: 

# We could either code up a chi square test using R as a programing language,
# or we could just use the version someone else wrote. 
# To do that we load another library (this time one that was pre-installed
# with R.)
library(MASS) 
chisq.test( femalePower   ) #From the MASS library.

#Which has a p-value of .7, suggesting that yes, they are independent. 

# So we can play the same game with Target:
mosaic( femaleTarget )       # looks good visually 
chisq.test( femaleTarget   ) # checks out formally

# But being the reasonable social scientist that you are, you know 
# about Simpson's paradox 
#       http://en.wikipedia.org/wiki/Simpson's_paradox
# and so you are concerned about these pairwise independence tests we
# just ran. Fear not, R can handle visualization of three (and higher)
# dimensional contingency tables. 

mosaic( femalePowerTarget )   # still looks good visually 

# For our formal test we need something other than chisq.test which
# just implements the standard test for two dimensional tables. This 
# is a three dimensional table, so we need a 3D test. Log-linear models 
# to the rescue: 
loglm( ~ Female + PowerGoal + Target, data=femalePowerTarget)

# And yes, our graphical intuition is correct. The p-value of 
# .84 is easily high enough. Our hypothesis of complete 
# independence among the three variables is retained.
# Counterbalancing worked for this experiment.

# *********************************************
# * WARNING : R does not replace common sense *
# *********************************************
# Now a quick warning. Note that:
chisq.test( femalePowerTarget )

# does NOT throw up an error. Instead it gives you the wrong answer,
# a p-value less than 10^(-7).

# What is going on? Look at the help page:
help(chisq.test)

# ANSWER: This function is *for* two by two tables. Give it something 
# that isn't a two by two table and it will 'helpfully' convert it to 
# a two by two table so it can give you an answer. It just happens to
# be a wrong answer if you do the wrong thing. 

# This is a general theme in R. It tries to be helpful, which is great
# if you know what you are doing, but it can burn you if you are 
# semi-blindly throwing data at functions. Read the help. Know what
# is going on. 

# *********************************
# * Numeric: Decimal Measurements *
# *********************************

# One thing you might want to look at is the distribution of 
# self-efficacy. My guess is that's in total_pre.

# So let's stare at a boxplot: 
boxplot(thermo11$total_pre)

# Well that's not very interesting. Let's decorate it so we 
# know what we are looking at. (You have to hit CTRL-ENTER
# for all three of the lines)
boxplot(thermo11$total_pre, 
        main="Self-Efficacy Pre-Test",
        ylab="Score")

# Perhaps you want to see how self-efficacy breaks down by
# gender
boxplot(total_pre ~ Female, data=thermo11,
        main="Self-Efficacy Pre-Test",
        ylab="Score", xlab="Female")

# So this time the ~ syntax returns, but as a way to split
# a numeric variable by a factor. This will be useful. 

# Now let's interpret the graph. It seems that the women had
# slightly lower self-efficacy at pre-test. Is this 
# difference in mean significant? 

# Let's do a formal test: 
t.test( total_pre ~ Female, data=thermo11)

# Read the output. Was the difference significant? 

# Now let's do the same thing for PowerGoal. Since the goal
# was randomized as part of the experimental design we 
# expect no difference in the box plots and no mean difference.

boxplot(total_pre ~ PowerGoal, data=thermo11,
        main="Self-Efficacy Pre-Test",
        ylab="Score", xlab="Power assigned as Goal")

# The spreads look different, but the centers look the same.

# Check with a formal test: 
t.test( total_pre ~ PowerGoal, data=thermo11)

# And we conclude that the means are not significantly 
# different. It seems like the randomization worked.

# Now a different question is how two numeric values are 
# related to each other. Let's consider pre and post test. 

plot(total_post ~ total_pre, data=thermo11)

# This scatter plot shows how the post test is predicted by
# the pre test. You can think of the syntax:
#      total_post ~ total_pre
# as
#      post test PREDICTED BY pre test

# Let's decorate so we know what we are looking at: 
plot(total_post ~ total_pre, data=thermo11,
     xlab="Pre test score", ylab="Post test score",
     main="Distribution of pre and post scores")

# Better. Now we might be interested in taking this prediction
# idea seriously and fitting a linear model: 
#    total_post = beta_0 + beta_1*total_pre + error
# where:
#    beta_0 is the intercept
#    beta_1 is the coefficent on the pre test
#    errror ~ Normal( 0, sigma2 )

# This is the "standard" line of best fit. In R you do
# this in two steps. (1) Fit the model. (2) Plot the resulting
# line. 
model_PrePost <- lm( total_post ~ total_pre, data=thermo11 )

# Here lm stands for Linear Model. The syntax is the same as 
# for the graph. Note that we don't have to specify the error 
# term or the beta coefficients. Those just fall straight out
# because R is helpful.

# Now let's look at the fit of this model by plotting the 
# line it predicts on the graph: 
plot(total_post ~ total_pre, data=thermo11,
     xlab="Pre test score", ylab="Post test score",
     main="Distribution of pre and post scores")
abline( model_PrePost )

# As expected, the line goes "through the middle" of the points.
# But how well does it fit? 

# Let's look at the error the line of best fit makes. These 
# are called residuals. Let's look at them: 
plot( resid(model_PrePost),  xlab="Student I", ylab="Residual",
      main="Residual Plot: total_post ~ total_pre")

# Now the residuals will always have an average of zero, so lets
# draw a horizontal line through zero to guide our eye:
abline(h=0)

# This looks like a textbook case of a model that fits well. 
# There is no discernible pattern in the residuals. 

# So now that we know that the model fits reasonably well, 
# let's look at the output more closely with the summary command:
summary(model_PrePost)

# And here we can that both the intercept and the coefficient for 
# the pretest (total_pre) are significantly different from zero.

# You may be more familiar with an ANOVA table. R makes those too:
anova( model_PrePost )

# So we can see that there is still a lot of variance left to explain.
# Let's try a few things. 

# Perhaps gender? 
model_PrePostFemale <- lm( total_post ~ total_pre + Female, data=thermo11 )

# So here I've just added a new predictor, Female. The model is now:
#    total_post = beta_0 + beta_1*total_pre + beta_2*Female + error
# where:
#    beta_0 is the intercept
#    beta_1 is the coefficent on the pre test
#    beta_2 is the coefficent on gender
#    errror ~ Normal( 0, sigma2 )

# And the summary shows: 
summary(model_PrePostFemale)

# Which is excerpted below: 
#  Coefficients:
#  Estimate Std. Error t value Pr(>|t|)    
#  (Intercept)  6.59224    1.12538   5.858 3.49e-08 ***
#  total_pre    0.64137    0.06675   9.609  < 2e-16 ***
#  FemaleFALSE  0.18321    0.66090   0.277    0.782    

# So now we have a new line that shows being male (FemaleFALSE) adds
# a little to your self-efficacy, but it isn't significantly different 
# from zero. 

# If you prefer the ANOVA way of looking at things, you can do that too: 
anova( model_PrePostFemale )

# And you can basically go from there. For example you could add in 
# additional fixed effects for having Power as a goal and being the Target.
# Jumping straight to the familiar ANOVA table:
anova( lm( total_post ~ total_pre + Female + PowerGoal + Target, data=thermo11 ) )

# We see that none of those variables really explain much variance. 

# That's it. You're done with this dataset. And this homework. 